﻿using Microsoft.WindowsAzure.MobileServices;
using MyBookshelf.Model;
using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace MyBookshelf.DataAccess
{
    public class DataManager
    {
        public MobileServiceClient Client { get; set; }

        public DataManager(string serviceUrl)
        {
            Client = new MobileServiceClient(serviceUrl);
        }

        public async Task SaveAsync<T>(IEnumerable<T> items) where T : TableBase
        {
            var table = this.Client.GetTable<T>();

            foreach (T item in items)
            {
                if (item.Id == null)
                {
                    item.Id = Guid.NewGuid().ToString("N");
                    await table.InsertAsync(item);
                }
                else
                {
                    await table.UpdateAsync(item);
                }
            }
        }

        public async Task<IEnumerable<T>> LoadAsync<T>(string userId) where T : TableBase
        {
            var table = this.Client.GetTable<T>();

            var query = await table.Where(t => t.UserId == userId).ToEnumerableAsync();

            return query;
        }
    }
}
